/* 
 * Copyright (C) 2014 Vincent Vansuyt
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import ij.IJ;
import ij.ImagePlus;
import ij.ImageStack;
import ij.WindowManager;
import ij.gui.GenericDialog;
import ij.plugin.PlugIn;
import ij.process.ImageProcessor;

import Fx_images.Fx_images;

/**
 * English description :
 *  Apply a selective median filter on image line (only for chrominance 
 * chanel a and b).
 * Filter is active when it detect a local minimum on luminance channel
 * 
 * Description en Français :
 *  Traite les lignes de l'image avec un filtre médian sélectif (filtre 
 * seulement les canaux de chrominance a et b).
 * Le filtre est actif sur les canaux a et b quand on détecte un
 *  minimum local important sur le canal L (luminance).
 * @author Vincent Vansuyt
 */
public class Line_Lab_selective_median_filtering_v1_00 implements PlugIn {
	/**
	 * Point d'entree du plugin
	 *
	 * @param string
	 */
	@Override
	public void run(String string) {
		IJ.log( "\\Clear" );
		IJ.log( "------- Line Lab selective median filtering - begin ------" );
		IJ.log( "Log folder : " + System.getProperty( "user.dir" ) );
		//imp IJ.getImage();
		ImagePlus imp = WindowManager.getCurrentImage();
		if (imp == null ) {
			IJ.noImage () ;
			return ;
		}				
		
		// − doit être stack
		int nNb_slices = imp.getNSlices();
		String strM = "Nb of slices = " + nNb_slices;
		IJ.log( strM );		
		int nWidth = imp.getWidth();
		int nHeight = imp.getHeight();
		String strNom_img_src = imp.getTitle();
		strM = "Image w:" + nWidth + " x h:" + nHeight;
		IJ.log( strM );
		if ( nNb_slices != 3 ) {
			strM = "Image must contain 3 stacks (L, a, b)";
			IJ.showMessage( "Warning", strM );
			IJ.log( strM );
			return ;
		}

		ImageStack imgstackIn = imp.getImageStack();
		ImageProcessor ipIn_L = imgstackIn.getProcessor( 0 + 1 );
		ImageProcessor ipIn_a = (imgstackIn.getProcessor( 1 + 1 ));
		ImageProcessor ipIn_b = (imgstackIn.getProcessor( 2 + 1 ));
		
		// Dialogue récupération des valeurs de sigma et la taille de la 
		//  fenêtre de filtre
		GenericDialog gd = new GenericDialog( "Color filtering" );
		gd.addMessage( "Selective median Lab filtering" );
		final int ncNb_decimales = 0;
		int nWinsz_x = 41;
		gd.addNumericField("Horizontal median length (pixels) : ", nWinsz_x, ncNb_decimales );
		double nDy_max = 40.0;
		gd.addNumericField("Luminance difference max before filtering : ", nDy_max, ncNb_decimales );
		gd.showDialog();
		if ( gd.wasCanceled() ) {
			IJ.log("Cancel" );
			IJ.log( "------- Line Lab selective median filtering - end ------" );
			return;
		}
		nWinsz_x = (int)gd.getNextNumber();
		IJ.log("Horizontal median length (pixels) : " + nWinsz_x );
		nDy_max = gd.getNextNumber();
		IJ.log("Luminance difference max before filtering : " + String.format( "%.2f", nDy_max ) );
		
		// Création de l'image-stack Lab résultat
		// Le canal L reste identique
		// Les canaux a et b sont filtrés avec un filtre bilateral basé sur la luminance
		final int ncNb_sclices = 3;		
		// "RGB Black" "8− bit White"
		IJ.log("strNom_img_src = " + strNom_img_src );
		String strNom_img_dest = strNom_img_src + "_Line_lab" +
				                   "_win_sz_X:" + String.format("%2d", nWinsz_x ) +
				                   "_dy_max:" + String.format("%.2f", nDy_max );
		IJ.log("strNom_img_dest = " + strNom_img_dest );
		ImagePlus imp_result = IJ.createImage( strNom_img_dest, "32-bit black" , nWidth , nHeight , ncNb_sclices );
		ImageStack imgstackOut = imp_result.getImageStack();
		ImageProcessor ipOut_L = imgstackOut.getProcessor( 0 + 1 );
		ImageProcessor ipOut_a = imgstackOut.getProcessor( 1 + 1 );
		ImageProcessor ipOut_b = imgstackOut.getProcessor( 2 + 1 );		
		
		IJ.log( "Parsing image..." );
		Fx_images fxImages = new Fx_images();
		int y_debut = 0;
		int y_fin = nHeight;
		for ( int y = y_debut; y < y_fin ; ++y ) {
			IJ.showProgress( y, nHeight );
			fxImages.calculer_ligne_median_selectif_float( y, ipIn_L, ipIn_a, ipOut_a, nWinsz_x, nDy_max );
			fxImages.calculer_ligne_median_selectif_float( y, ipIn_L, ipIn_b, ipOut_b, nWinsz_x, nDy_max );			
			for ( int x = 0; x < nWidth; ++x ) {
				ipOut_L.putPixelValue( x, y, ipIn_L.getPixelValue( x, y ));
			}
		}		
		imp_result.getProcessor().resetMinAndMax();
		imp_result.show();		
		IJ.log( "------- Line Lab selective median filtering - end ------" );
	}// run
}// Line_Lab_selective_median_filtering_v1_00